/**************************************************************************
*  Copyright (c) 1997-1999 by Joseph Parrello. All Rights Reserved.       *
*  Sites  : http://www.geocities.com/SiliconValley/Hills/8145 (USA)       *
*           http://JoParrello.freeweb.org (Europe)                        *
*           http://www.freeweb.org/computer/SourceNet/Owl (Italian)       *
*  E-Mail : joparrello@geocities.com, jofrank@geocities.com               *
*                                                                         *
*  Revision:   2.2  (September 1, 1999).                                  *
*                                                                         *
*  Definition of JPFlatComboBox class.                                    *
*                                                                         *
*  Read JPFLTBOX.TXT for use and license.                                 *
*                                                                         *
*  Only for OWL 5.02 and 5.01 users.                                      *
*  OWL and ObjectWindows are trademarks of Borland International.         *
*                                                                         *
*  This class and any related documentation is provided "as is" without   *
*  warranty of any kind, either express or implied, including, without    *
*  limitation, the implied warranties or merchantability, fitness for     *
*  a particular purpose. The entire risk arising out of use or            *
*  performance of this software remains with you.                         *
**************************************************************************/
#include <owl/pch.h>

#include "jpfltbox.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(JPFlatComboBox, TComboBox)
//{{JPFlatComboBoxRSP_TBL_BEGIN}}
  EV_NOTIFY_AT_CHILD(CBN_SETFOCUS, CbnSetFocus),
  EV_NOTIFY_AT_CHILD(CBN_KILLFOCUS, CbnKillFocus),
  EV_NOTIFY_AT_CHILD(CBN_CLOSEUP, CbnCloseUp),
  EV_NOTIFY_AT_CHILD(CBN_DROPDOWN, CbnDropDown),
  EV_NOTIFY_AT_CHILD(CBN_SELCHANGE, CbnSelChange),
  EV_WM_CTLCOLOR,
  EV_WM_PAINT,
  EV_WM_ENABLE,
  EV_WM_WINDOWPOSCHANGED,
  EV_WM_WINDOWPOSCHANGING,
  EV_WM_LBUTTONDOWN,
  EV_WM_LBUTTONUP,
  EV_WM_KILLFOCUS,
  EV_WM_SETFOCUS,
//{{JPFlatComboBoxRSP_TBL_END}}
END_RESPONSE_TABLE;

//{{JPFlatComboBox Implementation}}


JPFlatComboBox::JPFlatComboBox(TWindow* parent,
                               int id,
                               int x,
                               int y,
                               int w,
                               int h,
                               uint32 style,
                               uint textLimit,
                               TModule* module,
                               bool flatmode)
:
    TComboBox(parent, id, x, y, w, h, style, textLimit, module)
{
  // INSERT>> Your constructor code here.
   MouseInControl = false;
   MaxListWidth = w;
   hwndEdit = NULL;
   CurrentEditHeight = 0;
   RetrieveWindowsVersion();
   FlatMode = flatmode;
   Pressed = false;
   isEditFocused = false;
}


JPFlatComboBox::~JPFlatComboBox()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

}

/*******************************************************************************
*   This method draws the border of the combobox to make it "flat".            *
*******************************************************************************/
void JPFlatComboBox::RedrawBorder(void)
{
  TWindowDC mydc(GetHandle());
  TPoint point;
  GetCursorPos(point);
  ScreenToClient(point);
  TRect rect = GetClientRect();
  int thumbx = ::GetSystemMetrics(SM_CXHTHUMB);
  // If we are in Win31, we must paint the edit control and then draws our frame.
  if (IsInWin31)
      {
      TRect crect = rect;
      InflateRect(&crect, -2, -2);
      crect.right = crect.right - thumbx + 3;
      if (hwndEdit)
          {
          ::MoveWindow(hwndEdit, crect.left, crect.top, crect.right - crect.left, crect.bottom - crect.top, true);
          ::UpdateWindow(hwndEdit);
          }
      TBrush editbrush (TColor::SysWindow);
      mydc.FrameRect(crect, editbrush);
      mydc.RestoreBrush();
      }
  if (MouseInControl || isEditFocused)
   {              // we draw the "tracked" situation.
   TBrush brush (TColor::Sys3dFace);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   if (IsInWin31)
      rect.left = rect.right - thumbx + 3;
   else
      rect.left = rect.right - thumbx;
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, 0, 1);
   rect.right += 1;
   if (Pressed)
      TUIBorder(rect, TUIBorder::TStyle(TUIBorder::Recessed)).Paint(mydc);
   else
      {
      #if defined(BI_PLAT_WIN16)
      TUIBorder intborder(rect, TUIBorder::TEdge(TUIBorder::WndRaised), TUIBorder::Rect | TUIBorder::Adjust);
      #elif defined(BI_PLAT_WIN32)
      TUIBorder intborder(rect, TUIBorder::TEdge(TUIBorder::RaisedInner), TUIBorder::Rect | TUIBorder::Adjust);
      #endif
      intborder.Paint(mydc);
      }
   rect = GetClientRect();
   TUIBorder border(rect, TUIBorder::TStyle(TUIBorder::Recessed));
   border.Paint(mydc);
   }
else
   {              // We draw the "untracked" situation.
   TBrush brush (TColor::Sys3dFace);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   ::InflateRect (&rect, -1, -1);
   mydc.FrameRect(rect,brush);
   TBrush brush2 (TColor::SysWindow);
   mydc.FrameRect(rect,brush2);
   ::InflateRect (&rect, -1, -1);
   if (IsInWin31)
      {
      if (GetStyle() && CBS_DROPDOWNLIST == CBS_DROPDOWNLIST)
	 rect.left = rect.right - thumbx + 5;
      else
	 rect.left = rect.right - thumbx + 4;
      }
   else
      rect.left = rect.right - thumbx + 2;
   mydc.FrameRect(rect,brush);
#if defined(BI_PLAT_WIN16)
   if (::IsWindowEnabled(hwndEdit) == false)
     {
     int x1 = rect.left - 2;
     mydc.MoveTo(x1, rect.top);
     TPen pen (TColor::Sys3dFace);
     mydc.SelectObject(pen);
     mydc.LineTo(x1, rect.bottom);
     if (IsInWin31)
         {
         x1 = x1 + 1;
         mydc.MoveTo(x1, rect.top);
         mydc.LineTo(x1, rect.bottom);
         }
     mydc.RestorePen();
     }
#endif
   }
}

/*******************************************************************************
*   This method sets the height of the combobox according to current o.s.      *
*******************************************************************************/
void JPFlatComboBox::SetupWindow()
{
  TComboBox::SetupWindow();

  // INSERT>> Your code here.
  SetEditHeight();
}

/*******************************************************************************
*   This method sets the size of the list box, associates a tooltip to the     *
*   edit control and sets the background colour of the edit control.           *
*******************************************************************************/
HBRUSH JPFlatComboBox::EvCtlColor(HDC hDC, THandle hWndChild, uint ctlType)
{
  HBRUSH result;

  // INSERT>> Your code here.
  if (ctlType == CTLCOLOR_LISTBOX)
   {       // We change the width of the list box if MaxListWidth is unlike zero.
   RECT rect;
   ::GetWindowRect(hWndChild, &rect);
   if ((rect.right - rect.left) != MaxListWidth)
      ::SetWindowPos(hWndChild, HWND_TOPMOST, rect.left, rect.top, MaxListWidth, rect.bottom - rect.top, SWP_FRAMECHANGED);
   }
  else if (ctlType == CTLCOLOR_EDIT)
	 {  // We assign our string to the tooltip.
	 if (!hwndEdit)
	    InsertEditTooltip(hWndChild);
    /*if (!isEditFocused && FlatMode)
       {
       THandle hwndfocus = GetFocus();
       if (hwndfocus == hWndChild)
          {
          isEditFocused = true;
          if (!MouseInControl)
             Invalidate(false);
          }
       }*/
#if defined(BI_PLAT_WIN16)
	 else if (::IsWindowEnabled(hwndEdit) == false)
	      {
	      ::SetBkColor(hDC, TColor::Sys3dFace);
	      return (HBRUSH) TBrush(TColor::Sys3dFace);
	      }
#endif
	 }
  result = TComboBox::EvCtlColor(hDC, hWndChild, ctlType);
  return result;
}

/*******************************************************************************
*   This method paints the combobox and then the flat borders.                 *
*******************************************************************************/
void JPFlatComboBox::EvPaint()
{
    TComboBox::EvPaint();

  // INSERT>> Your code here.
  if (FlatMode)
     RedrawBorder();
}

/*******************************************************************************
*   This method assign our resource string to the tooltip.                     *
*******************************************************************************/
void JPFlatComboBox::InsertEditTooltip(HWND editHwnd)
{
  TWindow* tWin = GetParentO();
  if (tWin)
     {
     TGadgetWindow* gWin = TYPESAFE_DOWNCAST(tWin, TGadgetWindow);
     if (gWin)
         {
         TTooltip* tTip = gWin->GetTooltip();
         string text = LoadInternalString(GetId());
         if (tTip && text.length())
            {
            TToolInfo toolInfo(gWin->GetHandle(), editHwnd);
            #if defined(BI_PLAT_WIN32)
            toolInfo.SetText(text.c_str(), false);
            #elif defined(BI_PLAT_WIN16)
            toolInfo.SetText(text.c_str(), true);
            #endif
            if (tTip->AddTool(toolInfo))
               {   // The edit control finally has a tooltip!!!
               hwndEdit = editHwnd;
               if (IsInWin31)
             Invalidate(false);
               }
            }
         else
            {    // We assume the edit control has no tooltip.
            if (text.length() == 0)
               hwndEdit = editHwnd;
            }
         }
     }
}

/*******************************************************************************
*   This method checks if mouse is inside our combobox.                        *
*   Then we draw the borders of the combobox according to this checking.       *
*******************************************************************************/
bool JPFlatComboBox::IdleAction(long idleCount)
{
  // INSERT>> Your code here.
  if (idleCount == 0 && IsWindowEnabled())
     {
     TPoint point;
     GetCursorPos(point);
     ScreenToClient(point);
     TRect rect = GetClientRect();
     if (PtInRect(&rect, point))
         {
         if (MouseInControl == false)
            {
            MouseInControl = true;
            if (FlatMode && !isEditFocused)
               Invalidate(false);
            }
         }
     else
         {
         if (MouseInControl == true && !Pressed)
            {
            MouseInControl = false;
            if (FlatMode && !isEditFocused)
               Invalidate(false);
            }
         }
     }
  return false;
}

/*******************************************************************************
*   This method retrieves the resource string for the edit control (it's the   *
*   same as the combobox's resource string).                                   *
*******************************************************************************/
string JPFlatComboBox::LoadInternalString(uint itemId)
{
   char tempstr[256], szBuf[256];
   char* ptr = NULL, c = '\n';
   strcpy(szBuf,"");
   strcpy(tempstr,"");
   int len = GetModule()->LoadString(itemId, tempstr, sizeof(tempstr));
   if (len > 0)
      ptr = strchr(tempstr, c);
   if (ptr)
      strncpy(szBuf, ptr + 1, min(len, (int) sizeof(szBuf)));
   else
      {
      if (GetHandle())
          {
          TWindow* parent= GetParentO();
          TDecoratedFrame* frame= parent ? TYPESAFE_DOWNCAST(parent, TDecoratedFrame) : 0;
          while (parent && !frame)
               {
               parent = parent->GetParentO();
               if (parent)
                 frame = TYPESAFE_DOWNCAST(parent, TDecoratedFrame);
               }
          if (frame)
              frame->GetHintText(itemId, szBuf, sizeof(szBuf), htTooltip);
          else if (len > 0)
             strcpy(szBuf, tempstr);
          }
      }
   return string(szBuf);
}

/*******************************************************************************
*   This method sets the maximum width of the list box.                        *
*******************************************************************************/
int JPFlatComboBox::SetListBoxWidth(uint newwidth)
{
  MaxListWidth = newwidth;
  return MaxListWidth;
}

/*******************************************************************************
*   This method retrieves the maximum width of the list box.                   *
*******************************************************************************/
uint JPFlatComboBox::GetListBoxWidth() const
{
  return (uint) MaxListWidth;
}

/*******************************************************************************
*   This method sets the maximum width of the list box.                        *
*******************************************************************************/
void JPFlatComboBox::SetFlatMode(bool enable)
{
  if (enable != FlatMode)
     {
     FlatMode = enable;
     Invalidate(true);
     }
}

/*******************************************************************************
*   This method retrieves the maximum width of the list box.                   *
*******************************************************************************/
bool JPFlatComboBox::IsFlatModeEnabled() const
{
  return FlatMode;
}

// If the mouse is over the Combo-Box and we want to disable it, we must paint
// the normal combo-box before disabling it.
void JPFlatComboBox::EvEnable(bool enabled)
{
    TComboBox::EvEnable(enabled);

  // INSERT>> Your code here.
  if (!enabled && MouseInControl && FlatMode)
     {
     MouseInControl = false;
     Invalidate(false);
     }
}

void JPFlatComboBox::SetWindowFont(HFONT font, bool redraw)
{
  if (GetHandle())
     {
     SendMessage(WM_SETFONT, TParam1(font), redraw);
     TWindow* tWin = GetParentO();
     if (tWin)
         {
         TGadgetWindow* gWin = TYPESAFE_DOWNCAST(tWin, TGadgetWindow);
         if (gWin)
            gWin->LayoutSession();
         }
     }
}


void JPFlatComboBox::EvWindowPosChanged(WINDOWPOS far& windowPos)
{
    TComboBox::EvWindowPosChanged(windowPos);

  // INSERT>> Your code here.
  SetEditHeight();
  Invalidate(true);
}

void JPFlatComboBox::SetEditHeight()
{
  int Height = GetItemHeight(-1);
  if (Height != CurrentEditHeight)
     {
     if (IsInWin31)
	     Height = Height + 3;
     else Height = Height - 3;
     CurrentEditHeight = Height;
     SetItemHeight(-1, Height);
     }
}

void JPFlatComboBox::RetrieveWindowsVersion()
{
   IsInWin31 = false;

#if defined(BI_PLAT_WIN16)
   #ifndef SPI_GETDRAGFULLWINDOWS
   #define SPI_GETDRAGFULLWINDOWS 38
   #endif
   
   BOOL dfw;   
   // if "result" is true, we are under Win95/98/NT4. We need to know
   // if we are under Win95/98/NT4 as these operating systems draw 3d
   // comboboxes. Windows 3.1x and NT 3.xx don't draw 3d comboboxes.
   bool result = (bool) ::SystemParametersInfo(SPI_GETDRAGFULLWINDOWS, 0, &dfw, 0);
   DWORD dwVersion = ::GetVersion();
   bool isWoW = ToBool(::GetWinFlags() & 0x4000);  // Dos Box under WinNT?
   if ((LOBYTE(LOWORD(dwVersion)) == 3) && (HIBYTE(LOWORD(dwVersion)) < 95)
	&& (!isWoW || (isWoW && !result)))
      IsInWin31 = true;
#elif defined(BI_PLAT_WIN32)
   DWORD dwVersion = ::GetVersion();
   if (LOBYTE(LOWORD(dwVersion)) < 4)
      IsInWin31 = true;
#endif
}

void JPFlatComboBox::EvLButtonUp(uint modKeys, TPoint& point)
{
    TComboBox::EvLButtonUp(modKeys, point);

  // INSERT>> Your code here.
  if (FlatMode && (point.x == (int) -1) && (point.y == (int) -1))
       {
       Pressed = false;
       isEditFocused = false;
       Invalidate(false);
       }
}

TResult JPFlatComboBox::EvCommand(uint id, THandle hWndCtl, uint notifyCode)
{
  TResult result;

  result = TComboBox::EvCommand(id, hWndCtl, notifyCode);

  // INSERT>> Your code here.
  if (!isEditFocused && FlatMode && hWndCtl == hwndEdit && notifyCode == EN_SETFOCUS)
     {
     isEditFocused = true;
     if (!MouseInControl)
        Invalidate(false);
     }
  return result;
}

void JPFlatComboBox::CbnCloseUp()
{
  Pressed = false;
  if (!isEditFocused)
     GetApplication()->GetMainWindow()->GetClientWindow()->SetFocus();
  Invalidate(false);
}

void JPFlatComboBox::CbnDropDown()
{
  if (FlatMode)
     {
     Pressed = true;
     Invalidate(false);
     }
}

void JPFlatComboBox::CbnSetFocus()
{
  if (!isEditFocused && FlatMode)
     {
     isEditFocused = true;
     if (!MouseInControl)
        Invalidate(false);
     }
}

void JPFlatComboBox::CbnKillFocus()
{
  if (FlatMode)
     {
     Pressed = false;
     isEditFocused = false;
     Invalidate(false);
     }
}

void JPFlatComboBox::CbnSelChange()
{
  if (FlatMode)
     {
     Pressed = false;
     isEditFocused = false;
     GetApplication()->GetMainWindow()->GetClientWindow()->SetFocus();
     Invalidate(false);
     }
}

